﻿/*
 * 
 * 
 * ASP.net MonsterID HttpHandler
 * 
 * written by Alexander Schuc <aschuc@furrred.net>
 * 
 * licensed under Ms-PL
 * http://www.microsoft.com/resources/sharedsource/licensingbasics/permissivelicense.mspx
 * 
 * 
 * Based upon PHP implementation from Andreas Gohr
 * the graphics are from this guy too ;)
 * http://www.splitbrain.org/projects/monsterid
 * 
 * Idea for an indepentend ASP.net HttpHandler by Mads Kristensen
 * http://blog.madskristensen.dk/post/Cool-projects-that-don’t-exist.aspx
 * 
 * ----
 * 
 * Usage:
 * 
 * - Put MonsterID.cs into App_Code
 * - Copy MonsterID into App_Data
 *   Cached images are stored in App_Data/MonsterID/cache/[size]/Monster[seed].png
 * 
 * - Register the HttpHandler in your web.config
 * 
 * <httpHandlers>
 *			<add verb="GET" path="MonsterID.axd" type="furred.MonsterID" validate="false"/>
 *	</httpHandlers>
 *	
 * - Use it.. (something like that: <img src="MonsterID.axd?seed=blablabla&size=80" />)
 * 
 * ----
 * 
 * v0.2 - 01.08.2007 - security fix ;) - added size limit
 * v0.1 - 31.07.2007 - first release :D 
 * 
 */

using System;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Web.Security;
using System.Globalization;

using System.Drawing;
using System.IO;

namespace furred
{
	[WebService(Namespace = "http://net.furred.www/MonsterID")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	public class MonsterID : IHttpHandler
	{
		protected static readonly string cachePath = "~/App_Data/MonsterID/cache/{0}";
		protected static readonly string partsPath = "~/App_Data/MonsterID/parts";

		protected static readonly string[] parts = new string[] { "legs", "hair", "arms", "body", "eyes", "mouth" };
		protected static readonly int[] partcount = new int[] { 5, 5, 5, 15, 15, 10 };
		protected static readonly int maxSize = 120;

		public void ProcessRequest(HttpContext context)
		{
			
			HttpRequest request = context.Request;

			string seedSource;
			int seed;
			int size;

			if (!string.IsNullOrEmpty(request.QueryString["seed"]))
			{
				seedSource = FormsAuthentication.HashPasswordForStoringInConfigFile(request.QueryString["seed"], "md5");

				if (!int.TryParse(seedSource.Substring(0, 6), NumberStyles.HexNumber,
							CultureInfo.CurrentCulture.NumberFormat, out seed))
					seed = (int)DateTime.Now.Ticks;
			}
			else
				seed = (int)DateTime.Now.Ticks;

			if (!int.TryParse(request.QueryString["size"], out size) || size < 1 || size >= maxSize)
				size = 120;

			context.Response.ContentType = "image/png";
			context.Response.Cache.SetCacheability(HttpCacheability.Public);
			context.Response.Cache.SetETag(string.Format("{0}:{1}", seed, size));
			context.Response.WriteFile(GetMonster(seed, size));
		}

		private string GetMonster(int seed, int size)
		{
			string filename = GetMonsterFilename(seed, size);
			if (!File.Exists(filename))
				CreateMonster(seed, size);

			return filename;
		}

		protected string GetMonsterFilename(int seed, int size)
		{
			string filename = Path.Combine(
					HttpContext.Current.Server.MapPath(string.Format(cachePath, size)),
					string.Format("Monster{0}.png", seed));
			return filename;
		}

		protected void CreateMonster(int seed, int size)
		{
			string sourcedir = HttpContext.Current.Server.MapPath(partsPath);

			Random rnd = new Random(seed);

			int[] currentParts = new int[parts.Length];

			for (int i = 0; i < currentParts.Length; i++)
			{
				currentParts[i] = rnd.Next(1, partcount[i]);
			}

			using (Bitmap bmp = new Bitmap(size, size))
			{
				Bitmap overlay;
				Graphics gfx = Graphics.FromImage(bmp);

				gfx.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceOver;
				gfx.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;

				for (int i = 0; i < currentParts.Length; i++)
				{
					using (overlay = new Bitmap(Path.Combine(sourcedir,
						string.Format("{0}_{1}.png", parts[i], currentParts[i]))))
					{
						gfx.DrawImage(overlay, new Rectangle(new Point(0), bmp.Size),
							new Rectangle(new Point(0), overlay.Size), GraphicsUnit.Pixel);
					}
				}

				gfx.Dispose();
				gfx = null;

				string filename = GetMonsterFilename(seed, size);
				string path = Path.GetDirectoryName(filename);
				if (!Directory.Exists(path))
					Directory.CreateDirectory(path);

				bmp.Save(filename, System.Drawing.Imaging.ImageFormat.Png);
			}
		}

		public bool IsReusable
		{
			get
			{
				return true;
			}
		}
	}
}
